@@ -25,7 +25,7 @@ for e in `find $problems -regex ".*[.]wcsp" -print | sort` ; do
 
     if [[ -e $ubfile ]] ; then
 	    ub=`cat $ubfile`
-	    ub=`expr $ub \* $K`
+#	    ub=`expr $ub \* $K`
         echo -n $ub > out
     else
         ub=
@@ -42,7 +42,7 @@ for e in `find $problems -regex ".*[.]wcsp" -print | sort` ; do
 #    cat outsolver | awk -v UB=$ub -f ./misc/script/runall.awk >> out ; cat out
 # UNCOMMENT PREVIOUS *OR* NEXT TWO LINES
     (/usr/bin/time -f "%U user %S sys" $solver $file.wcsp "$@" -C=$K >> outsolver) 2> usedtime
-    cat outsolver | awk -v UB=-1 -f ./misc/script/runall.awk >> out ; cat out
+    cat outsolver | awk -v UB=$ub -f ./misc/script/runall.awk >> out ; cat out
 
     cat usedtime | awk '/ user /{ printf("%.2f",0.0+$1+$3); }'
    
