@@ -465,7 +465,7 @@ protected:
     DLink<ConstraintLink>** links;
 
     vector<EnumeratedVariable::iterator> it_values; // used by firstlex/nextlex and for separator decomposition tests
-    String evalTuple;
+    Tuple evalTuple;
 
 public:
     AbstractNaryConstraint(WCSP* wcsp, EnumeratedVariable** scope_in, int arity_in)
@@ -482,7 +482,7 @@ public:
             scope[i] = var;
             scope_dac[i] = var;
             links[i] = var->link(this, i);
-            evalTuple.append(1, CHAR_FIRST);
+            evalTuple.push_back(0);
         }
         setDACScopeIndex();
     }
@@ -492,7 +492,12 @@ public:
     {
     }
 
-    virtual ~AbstractNaryConstraint() {}
+    virtual ~AbstractNaryConstraint()
+    {
+        delete[] scope;
+        delete[] scope_dac;
+        delete[] links;
+    }
 
     int arity() const FINAL { return arity_; }
     Long getDomainInitSizeProduct(); // warning! return LONGLONG_MAX if overflow occurs
@@ -559,12 +564,12 @@ public:
         }
     }
 
-    virtual Cost eval(const String& t)
+    virtual Cost eval(const Tuple& t)
     {
         cout << "dummy eval on: " << *this << endl;
         return MIN_COST;
     }
-    //    virtual void insertTuple( String t, Cost c, EnumeratedVariable** scope_in ) { }
+    //    virtual void insertTuple( Tuple t, Cost c, EnumeratedVariable** scope_in ) { }
 
     int getSmallestVarIndexInScope(int forbiddenScopeIndex) FINAL
     {
@@ -643,7 +648,7 @@ public:
     }
 
     void firstlex();
-    bool nextlex(String& t, Cost& c);
+    bool nextlex(Tuple& t, Cost& c);
 
     void projectNaryBeforeSearch();
     // USE ONLY DURING SEARCH when less than three unassigned variables remain:
