@@ -68,6 +68,8 @@ protected:
     void extendY(Value value, Cost cost) { extend(y, value, cost, deltaCostsY); }
 
 public:
+    vector<Cost> trwsM;
+
     BinaryConstraint(WCSP* wcsp, EnumeratedVariable* xx, EnumeratedVariable* yy, vector<Cost>& tab);
 
     BinaryConstraint(WCSP* wcsp);
@@ -87,6 +89,39 @@ public:
         assert(res >= MIN_COST);
         return res;
     }
+    Cost getCostTRWS(Value vx, Value vy) const
+    {
+        unsigned int ix = x->toIndex(vx);
+        unsigned int iy = y->toIndex(vy);
+        return costs[ix * sizeY + iy] - (deltaCostsX[ix] + deltaCostsY[iy]);
+    }
+    void projectTRWS(EnumeratedVariable* var, Value value, Cost cost)
+    {
+        vector<StoreCost>& deltaCosts = (var == x) ? deltaCostsX : deltaCostsY;
+        deltaCosts[var->toIndex(value)] += cost;
+        var->project(value, cost, true);
+    }
+    Cost normalizeTRWS()
+    {
+        Cost minCost = numeric_limits<Cost>::max();
+        for (EnumeratedVariable::iterator xIter = x->begin(); xIter != x->end(); ++xIter) {
+            unsigned int ix = x->toIndex(*xIter);
+            for (EnumeratedVariable::iterator yIter = y->begin(); yIter != y->end(); ++yIter) {
+                unsigned int iy = y->toIndex(*yIter);
+                minCost = min<Cost>(minCost, costs[ix * sizeY + iy] - (deltaCostsX[ix] + deltaCostsY[iy]));
+            }
+        }
+        if (minCost != MIN_COST) {
+            for (EnumeratedVariable::iterator xIter = x->begin(); xIter != x->end(); ++xIter) {
+                unsigned int ix = x->toIndex(*xIter);
+                for (EnumeratedVariable::iterator yIter = y->begin(); yIter != y->end(); ++yIter) {
+                    unsigned int iy = y->toIndex(*yIter);
+                    costs[ix * sizeY + iy] -= minCost;
+                }
+            }
+        }
+        return minCost;
+    }
 
     Cost getCost(EnumeratedVariable* xx, EnumeratedVariable* yy, Value vx, Value vy)
     {
@@ -198,6 +233,20 @@ public:
         }
     }
 
+    Cost getMaxFiniteCost()
+    {
+        Cost ub = wcsp->getUb();
+        Cost maxcost = MIN_COST;
+        for (EnumeratedVariable::iterator iterX = x->begin(); iterX != x->end(); ++iterX) {
+            for (EnumeratedVariable::iterator iterY = y->begin(); iterY != y->end(); ++iterY) {
+                Cost cost = getCost(*iterX, *iterY);
+                if (cost < ub && cost > maxcost)
+                    maxcost = cost;
+            }
+        }
+        return maxcost;
+    }
+
     void setInfiniteCost(Cost ub)
     {
         Cost mult_ub = ((ub < (MAX_COST / MEDIUM_COST)) ? (max(LARGE_COST, ub * MEDIUM_COST)) : ub);
@@ -213,7 +262,7 @@ public:
         }
     }
 
-    Cost evalsubstr(const String& s, Constraint* ctr) FINAL
+    Cost evalsubstr(const Tuple& s, Constraint* ctr) FINAL
     {
         Value vals[2];
         int count = 0;
@@ -222,7 +271,7 @@ public:
             EnumeratedVariable* var = (EnumeratedVariable*)getVar(i);
             int ind = ctr->getIndex(var);
             if (ind >= 0) {
-                vals[i] = var->toValue(s[ind] - CHAR_FIRST);
+                vals[i] = var->toValue(s[ind]);
                 count++;
             }
         }
@@ -231,7 +280,13 @@ public:
         else
             return MIN_COST;
     }
-    Cost evalsubstr(const String& s, NaryConstraint* ctr) FINAL { return evalsubstr(s, (Constraint*)ctr); } // NaryConstraint class undefined
+    Cost evalsubstr(const Tuple& s, NaryConstraint* ctr) FINAL { return evalsubstr(s, (Constraint*)ctr); } // NaryConstraint class undefined
+
+    void resetSupports()
+    {
+        supportX.assign(supportX.size(), y->getInf());
+        supportY.assign(supportY.size(), x->getInf());
+    }
 
     Value getSupport(EnumeratedVariable* var, Value v)
     {
@@ -262,16 +317,15 @@ public:
         yvar = y;
     }
 
-    bool next(String& t, Cost& c)
+    bool next(Tuple& t, Cost& c)
     {
-        Char tch[3];
+        Tuple tch(2, 0);
         if (itvx != xvar->end()) {
             unsigned int ix = xvar->toIndex(*itvx);
-            tch[0] = ix + CHAR_FIRST;
+            tch[0] = ix;
             if (itvy != yvar->end()) {
                 unsigned int iy = yvar->toIndex(*itvy);
-                tch[1] = iy + CHAR_FIRST;
-                tch[2] = '\0';
+                tch[1] = iy;
                 t = tch;
                 c = getCost(xvar, yvar, *itvx, *itvy);
                 ++itvy;
@@ -286,19 +340,19 @@ public:
     }
 
     void firstlex() { first(); }
-    bool nextlex(String& t, Cost& c) { return next(t, c); }
+    bool nextlex(Tuple& t, Cost& c) { return next(t, c); }
 
-    void setTuple(const String& t, Cost c) FINAL
+    void setTuple(const Tuple& t, Cost c) FINAL
     {
-        Value v0 = x->toValue(t[0] - CHAR_FIRST);
-        Value v1 = y->toValue(t[1] - CHAR_FIRST);
+        Value v0 = x->toValue(t[0]);
+        Value v1 = y->toValue(t[1]);
         setcost(v0, v1, c);
     }
 
-    void addtoTuple(const String& t, Cost c) FINAL
+    void addtoTuple(const Tuple& t, Cost c) FINAL
     {
-        Value v0 = x->toValue(t[0] - CHAR_FIRST);
-        Value v1 = y->toValue(t[1] - CHAR_FIRST);
+        Value v0 = x->toValue(t[0]);
+        Value v1 = y->toValue(t[1]);
         addcost(v0, v1, c);
     }
 
@@ -328,8 +382,10 @@ public:
             supportX.resize(sizeX);
         if (sizeY > supportY.size())
             supportY.resize(sizeY);
-        if (sizeX * sizeY > costs.size())
-            costs.resize(sizeX * sizeY, StoreCost(MIN_COST));
+        if (max(sizeX, sizeY) > trwsM.size())
+            trwsM.resize(max(sizeX, sizeY), MIN_COST);
+        if ((unsigned long)sizeX * (unsigned long)sizeY > costs.size())
+            costs.resize((size_t)sizeX * (size_t)sizeY, StoreCost(MIN_COST));
         linkX->removed = true;
         linkY->removed = true;
         linkX->content.constr = this;
@@ -381,6 +437,14 @@ public:
             else
                 x->queueAC();
         }
+        if (ToulBar2::FullEAC) {
+            reviseEACGreedySolution();
+            for (ConstraintList::iterator iter = x->getConstrs()->begin(); iter != x->getConstrs()->end(); ++iter) {
+                if ((*iter).constr->isTernary() && (*iter).constr->getIndex(y) >= 0) {
+                    (*iter).constr->reviseEACGreedySolution(); // all ternary cost functions including this current binary cost function need to be revised
+                }
+            }
+        }
     }
     void remove(int varIndex)
     {
@@ -439,6 +503,29 @@ public:
         }
     }
 
+    bool checkEACGreedySolution(int index = -1, Value a = 0) FINAL
+    {
+        assert(x->canbe((getIndex(x) == index) ? a : x->getSupport()));
+        assert(x->getCost((getIndex(x) == index) ? a : x->getSupport()) == MIN_COST);
+        assert(y->canbe((getIndex(y) == index) ? a : y->getSupport()));
+        assert(y->getCost((getIndex(y) == index) ? a : y->getSupport()) == MIN_COST);
+        return (getCost((getIndex(x) == index) ? a : x->getSupport(), (getIndex(y) == index) ? a : y->getSupport()) == MIN_COST);
+    }
+
+    bool reviseEACGreedySolution(int index = -1, Value a = 0) FINAL
+    {
+        bool result = checkEACGreedySolution(index, a);
+        if (!result) {
+            if (index >= 0)
+                getVar(index)->unsetFullEAC();
+            else {
+                x->unsetFullEAC();
+                y->unsetFullEAC();
+            }
+        }
+        return result;
+    }
+
     void fillEAC2(int varIndex)
     {
         assert(!isDuplicate());
@@ -446,7 +533,7 @@ public:
             if (varIndex == 0) {
                 assert(y->canbe(y->getSupport()));
                 unsigned int yindex = y->toIndex(y->getSupport());
-                if (x->cannotbe(supportY[yindex]) || x->getCost(supportY[yindex]) > MIN_COST || getCost(supportY[yindex], y->getSupport()) > MIN_COST || (ToulBar2::vacValueHeuristic && Store::getDepth() < ToulBar2::vac)) {
+                if (x->cannotbe(supportY[yindex]) || x->getCost(supportY[yindex]) > MIN_COST || getCost(supportY[yindex], y->getSupport()) > MIN_COST || (ToulBar2::vacValueHeuristic && Store::getDepth() < abs(ToulBar2::vac))) {
                     y->queueEAC2();
                 }
             }
@@ -454,7 +541,7 @@ public:
             if (varIndex == 1) {
                 assert(x->canbe(x->getSupport()));
                 unsigned int xindex = x->toIndex(x->getSupport());
-                if (y->cannotbe(supportX[xindex]) || y->getCost(supportX[xindex]) > MIN_COST || getCost(x->getSupport(), supportX[xindex]) > MIN_COST || (ToulBar2::vacValueHeuristic && Store::getDepth() < ToulBar2::vac)) {
+                if (y->cannotbe(supportX[xindex]) || y->getCost(supportX[xindex]) > MIN_COST || getCost(x->getSupport(), supportX[xindex]) > MIN_COST || (ToulBar2::vacValueHeuristic && Store::getDepth() < abs(ToulBar2::vac))) {
                     x->queueEAC2();
                 }
             }
@@ -464,29 +551,41 @@ public:
     bool isEAC(int varIndex, Value a)
     {
         assert(!isDuplicate());
-        if (ToulBar2::QueueComplexity && varIndex == getDACScopeIndex())
+        if (ToulBar2::QueueComplexity && varIndex == getDACScopeIndex() && !ToulBar2::FullEAC)
             return true;
         if (varIndex == 0) {
-            unsigned int xindex = x->toIndex(a);
-            if (y->cannotbe(supportX[xindex]) || y->getCost(supportX[xindex]) > MIN_COST || getCost(a, supportX[xindex]) > MIN_COST) {
-                for (EnumeratedVariable::iterator iterY = y->begin(); iterY != y->end(); ++iterY) {
-                    if (y->getCost(*iterY) == MIN_COST && getCost(a, *iterY) == MIN_COST) {
-                        supportX[xindex] = *iterY;
-                        return true;
+            assert(y->canbe(y->getSupport()));
+            assert(y->getCost(y->getSupport()) == MIN_COST);
+            if (getCost(a, y->getSupport()) > MIN_COST) {
+                if (ToulBar2::FullEAC)
+                    x->unsetFullEAC();
+                unsigned int xindex = x->toIndex(a);
+                if (y->cannotbe(supportX[xindex]) || y->getCost(supportX[xindex]) > MIN_COST || getCost(a, supportX[xindex]) > MIN_COST) {
+                    for (EnumeratedVariable::iterator iterY = y->begin(); iterY != y->end(); ++iterY) {
+                        if (y->getCost(*iterY) == MIN_COST && getCost(a, *iterY) == MIN_COST) {
+                            supportX[xindex] = *iterY;
+                            return true;
+                        }
                     }
+                    return false;
                 }
-                return false;
             }
         } else {
-            unsigned int yindex = y->toIndex(a);
-            if (x->cannotbe(supportY[yindex]) || x->getCost(supportY[yindex]) > MIN_COST || getCost(supportY[yindex], a) > MIN_COST) {
-                for (EnumeratedVariable::iterator iterX = x->begin(); iterX != x->end(); ++iterX) {
-                    if (x->getCost(*iterX) == MIN_COST && getCost(*iterX, a) == MIN_COST) {
-                        supportY[yindex] = *iterX;
-                        return true;
+            assert(x->canbe(x->getSupport()));
+            assert(x->getCost(x->getSupport()) == MIN_COST);
+            if (getCost(x->getSupport(), a) > MIN_COST) {
+                if (ToulBar2::FullEAC)
+                    y->unsetFullEAC();
+                unsigned int yindex = y->toIndex(a);
+                if (x->cannotbe(supportY[yindex]) || x->getCost(supportY[yindex]) > MIN_COST || getCost(supportY[yindex], a) > MIN_COST) {
+                    for (EnumeratedVariable::iterator iterX = x->begin(); iterX != x->end(); ++iterX) {
+                        if (x->getCost(*iterX) == MIN_COST && getCost(*iterX, a) == MIN_COST) {
+                            supportY[yindex] = *iterX;
+                            return true;
+                        }
                     }
+                    return false;
                 }
-                return false;
             }
         }
         return true;
@@ -495,7 +594,7 @@ public:
     void findFullSupportEAC(int varIndex)
     {
         assert(!isDuplicate());
-        if (ToulBar2::QueueComplexity && varIndex == getDACScopeIndex())
+        if (ToulBar2::QueueComplexity && varIndex == getDACScopeIndex() && !ToulBar2::FullEAC)
             return;
         if (varIndex == 0)
             findFullSupportX();
@@ -515,7 +614,7 @@ public:
         }
     }
 
-    pair<pair<Cost, Cost>, pair<Cost, Cost> > getMaxCost(int varIndex, Value a, Value b);
+    pair<pair<Cost, Cost>, pair<Cost, Cost>> getMaxCost(int varIndex, Value a, Value b);
 
     double computeTightness()
     {
@@ -553,8 +652,9 @@ public:
 
     bool isFunctional(EnumeratedVariable* x, EnumeratedVariable* y, map<Value, Value>& functional);
 
-    void print(ostream& os);
+    virtual void print(ostream& os);
     void dump(ostream& os, bool original = true);
+    void dump_CFN(ostream& os, bool original = true);
     Long size() const FINAL { return (Long)sizeX * sizeY; }
     Long space() const FINAL { return (Long)sizeof(StoreCost) * sizeX * sizeY; }
 
