@@ -1,6 +1,5 @@
 #include "tb2globalconstr.hpp"
 #include "core/tb2wcsp.hpp"
-#include "core/tb2vac.hpp"
 #include "search/tb2clusters.hpp"
 
 GlobalConstraint::GlobalConstraint(WCSP* wcsp, EnumeratedVariable** scope_in, int arity_in, Cost defval)
@@ -104,14 +103,14 @@ void GlobalConstraint::print(ostream& os)
             os << *scope[i] << endl;
 }
 
-Cost GlobalConstraint::eval(const String& s)
+Cost GlobalConstraint::eval(const Tuple& s)
 {
 
     Cost tcost = evalOriginal(s);
-    for (unsigned int i = 0; i < s.length(); i++) {
+    for (unsigned int i = 0; i < s.size(); i++) {
         EnumeratedVariable* x = (EnumeratedVariable*)getVar(i);
         if (tcost < wcsp->getUb()) {
-            tcost -= deltaCost[i][x->toIndex(s[i] - CHAR_FIRST)];
+            tcost -= deltaCost[i][x->toIndex(s[i])];
         }
     }
     if (tcost < wcsp->getUb()) {
@@ -129,14 +128,16 @@ void GlobalConstraint::assign(int varIndex)
         nonassigned = nonassigned - 1;
         if (nonassigned == 0) {
             deconnect();
-            String t(arity_, CHAR_FIRST);
+            Tuple t(arity_, 0);
             for (int i = 0; i < arity_; i++) {
-                t[i] = getVar(i)->getValue() + CHAR_FIRST;
+                t[i] = ((EnumeratedVariable*)getVar(i))->toIndex(getVar(i)->getValue());
             }
             wcsp->revise(this);
             projectLB(eval(t));
         } else {
             pushAll();
+            if (ToulBar2::FullEAC)
+                reviseEACGreedySolution();
         }
     }
 }
@@ -271,7 +272,7 @@ void GlobalConstraint::propagateDAC()
     if (ToulBar2::verbose >= 3)
         cout << "propagateDAC for " << *this << endl;
 
-    vector<map<Value, Cost> > deltas;
+    vector<map<Value, Cost>> deltas;
     vector<int> vars;
     vector<int> rmv;
     //checkRemoved(rmv);
@@ -408,6 +409,8 @@ void GlobalConstraint::propagateNIC()
 
 bool GlobalConstraint::isEAC(int index, Value a)
 {
+    if (ToulBar2::FullEAC)
+        reviseEACGreedySolution(index, a);
     if (currentVar != index) {
         currentVar = index;
 
@@ -417,7 +420,7 @@ bool GlobalConstraint::isEAC(int index, Value a)
         checkRemoved(rmv);
 
         vector<int> support;
-        vector<map<Value, Cost> > deltas;
+        vector<map<Value, Cost>> deltas;
         for (set<int>::iterator i = fullySupportedSet[index].begin(); i != fullySupportedSet[index].end(); i++) {
             EnumeratedVariable* x = (EnumeratedVariable*)getVar(*i);
             if (x->unassigned() && (*i != index)) {
@@ -464,7 +467,7 @@ void GlobalConstraint::findFullSupportEAC(int index)
     {
         //needPropagateEAC = false;
 
-        vector<map<Value, Cost> > deltas;
+        vector<map<Value, Cost>> deltas;
         vector<int> vars;
         vector<int> rmv;
         //vars.push_back(index);
@@ -527,7 +530,7 @@ void GlobalConstraint::findFullSupport(int varindex, vector<int>& support, bool
         cout << "findFullSupport for variable " << varindex << endl;
     //wcsp->revise(this);
     EnumeratedVariable* var = (EnumeratedVariable*)getVar(varindex);
-    vector<map<Value, Cost> > deltas(support.size());
+    vector<map<Value, Cost>> deltas(support.size());
     int count = 0;
     for (vector<int>::iterator i = support.begin(); i != support.end(); i++, count++) {
         EnumeratedVariable* x = (EnumeratedVariable*)getVar(*i);
@@ -563,7 +566,7 @@ void GlobalConstraint::findFullSupport(int varindex, vector<int>& support, bool
     changeAfterProject(varindex, delta);
 
     //	map<Value,Cost> *E = new map<Value, Cost>[support.size()];
-    vector<map<Value, Cost> > E;
+    vector<map<Value, Cost>> E;
     E.resize(support.size());
 
     int index = 0;
