@@ -2395,7 +2395,7 @@ void TreeDecomposition::dump(Cluster* c)
     if (!c) {
         char tmpName[256];
         sprintf(tmpName, "%s.info", getWCSP()->getName().c_str());
-#ifdef WIN32
+#ifdef __WIN32__
         mkdir(tmpName);
 #else
         mkdir(tmpName, 0777);
