@@ -21,15 +21,15 @@ public:
     int n, m;
 
     bool connected();
-    void generateGlobalCtr(vector<int>& indexs, string globalname, Cost costMin = SMALL_COST, Cost costMax = MEDIUM_COST);
-    void generateNaryCtr(vector<int>& indexs, long nogoods, Cost costMin = SMALL_COST, Cost costMax = MEDIUM_COST);
-    void generateTernCtr(int i, int j, int k, long p, Cost costMin = SMALL_COST, Cost costMax = MEDIUM_COST);
-    void generateBinCtr(int i, int j, long p, Cost costMin = SMALL_COST, Cost costMax = MEDIUM_COST);
-    void generateSubModularBinCtr(int i, int j, Cost costMin = SMALL_COST, Cost costMax = MEDIUM_COST);
+    void generateGlobalCtr(vector<int>& indexs, string globalname, Cost costMin = SMALL_COST, Cost costMax = LARGE_COST);
+    void generateNaryCtr(vector<int>& indexs, long nogoods, Cost costMin = SMALL_COST, Cost costMax = LARGE_COST);
+    void generateTernCtr(int i, int j, int k, long p, Cost costMin = SMALL_COST, Cost costMax = LARGE_COST);
+    void generateBinCtr(int i, int j, long p, Cost costMin = SMALL_COST, Cost costMax = LARGE_COST);
+    void generateSubModularBinCtr(int i, int j, Cost costMin = SMALL_COST, Cost costMax = LARGE_COST);
     void Input(int in_n, int in_m, vector<int>& p, bool forceSubModular = false, string globalname = "");
 
-    void ini(vector<int>& index, int arity);
-    long toIndex(vector<int>& index);
+    void ini(vector<int>& index, int arity, int n);
+    long long toIndex(vector<int>& index);
     int inc(vector<int>& index, int i);
     bool inc(vector<int>& index);
 };
