@@ -301,7 +301,7 @@ const zone RandomClusterChoice::getNeighborhood(size_t neighborhood_size, zone z
     return zone();
 }
 
-const bool RandomClusterChoice::incrementK()
+bool RandomClusterChoice::incrementK()
 {
     if (file.size() == 0) {
         file = clusters;
@@ -500,7 +500,7 @@ const zone ParallelRandomClusterChoice::SlaveGetNeighborhood(unsigned int Curren
     return insideHeuristic->getNeighborhood(NeighborhoodSize, z);
 }
 
-const bool ParallelRandomClusterChoice::incrementK()
+bool ParallelRandomClusterChoice::incrementK()
 {
     if (file.size() == 0) {
         file = clusters;
